﻿//-----------------------------------------------------------------------
// <copyright file="AttributeType.cs" company="Craig Longford (deltawolf7@gmail.com)">
//     Copyright (c) Craig Longford. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

namespace Delta.Mercury.Business.Commerce
{
    using System;
    using Delta.Data.DataExpress;
    using Delta.Mercury.Business.Exceptions;

    /// <summary>
    /// Attribute type class
    /// </summary>
    [Serializable]
    public class AttributeType : IdentifierBase, IDataServicable
    {
        #region Members
        /// <summary>
        /// Attribute type name
        /// </summary>
        private string name;
        #endregion

        #region Constructors
        /// <summary>
        /// Initializes a new instance of the AttributeType class.
        /// </summary>
        public AttributeType()
            : this(string.Empty)
        {
        }

        /// <summary>
        /// Initializes a new instance of the AttributeType class.
        /// </summary>
        /// <param name="name">Attribute name</param>
        public AttributeType(string name)
            : base()
        {
            this.name = name;
        }
        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets the attribute name
        /// </summary>
        public string Name
        {
            get
            {
                return this.name;
            }

            set
            {
                if (value.Length < 1)
                {
                    throw new InvalidLengthException("Attribute type name must be a minimum of 1 characters.");
                }

                if (value.Length > 30)
                {
                    throw new InvalidLengthException("Attribute type name must be a maximum of 30 characters.");
                }

                this.name = value;
            }
        }
        #endregion

        #region Static Load Methods
        /// <summary>
        /// Load attribute type
        /// </summary>
        /// <param name="guid">Unique identifier</param>
        /// <returns>Arrtibute type</returns>
        public static AttributeType Load(Guid guid)
        {
            Parameters parameters = new Parameters();
            parameters.Add(new Parameter("Guid", guid));
            return DataAccess.GetSingle<AttributeType>("LoadAttributeType", System.Data.CommandType.StoredProcedure, parameters);
        }
        #endregion

        #region Save Methods
        /// <summary>
        /// Save attribute type
        /// </summary>
        public void Save()
        {
            DataAccess.NonQuery("SaveAttributeType", System.Data.CommandType.StoredProcedure, GetSaveParameters());
        }
        #endregion

        #region Delete Methods
        /// <summary>
        /// Delete attribute type
        /// </summary>
        public void Delete()
        {
            Parameters parameters = new Parameters();
            parameters.Add(new Parameter("Guid", this.Guid));
            DataAccess.NonQuery("DeleteAttributeType", System.Data.CommandType.StoredProcedure, parameters);
        }
        #endregion

        #region Fill Method
        /// <summary>
        /// Fill method
        /// </summary>
        /// <param name="reader">Data reader</param>
        public void Fill(System.Data.IDataReader reader)
        {
            this.name = Helper.ToString(reader["Name"]);
            this.Guid = Helper.ToGuid(reader["Guid"]);
        }
        #endregion

        #region Get Save Parameter
        /// <summary>
        /// Get save parameters
        /// </summary>
        /// <returns>Parameter collection</returns>
        public Parameters GetSaveParameters()
        {
            Parameters parameters = new Parameters();
            parameters.Add(new Parameter("Guid", this.Guid));
            parameters.Add(new Parameter("Name", this.Name));
            return parameters;
        }
        #endregion
    }
}
