﻿//-----------------------------------------------------------------------
// <copyright file="ContactType.cs" company="Craig Longford (deltawolf7@gmail.com)">
//     Copyright (c) Craig Longford. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

namespace Delta.Mercury.Business.User
{
    using System;
    using Delta.Data.DataExpress;
    using Delta.Mercury.Business.Exceptions;

    /// <summary>
    /// Representaion of a type of contact.
    /// </summary>
    [Serializable]
    public class ContactType : IdentifierBase, IDataServicable
    {
        #region Members
        /// <summary>
        /// Contact type name
        /// </summary>
        private string name;
        #endregion

        #region Constructors
        /// <summary>
        /// Initializes a new instance of the ContactType classs.
        /// </summary>
        public ContactType() 
            : this(string.Empty)
        {           
        }

        /// <summary>
        /// Initializes a new instance of the ContactType classs.
        /// </summary>
        /// <param name="name">Name of the contact type</param>
        public ContactType(string name)
            : base()
        {
            this.name = name;
            this.SortOrder = 0;
        }
        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets the name of the contact type.
        /// </summary>
        public virtual string Name
        {
            get
            {
                return this.name;
            }

            set
            {
                if (value.Length < 1)
                {
                    throw new InvalidLengthException("Contact type name must be a minimum of 1 characters.");
                }

                if (value.Length > 20)
                {
                    throw new InvalidLengthException("Contact type name must be a maximum of 20 characters.");
                }

                this.name = value;
            }
        }

        /// <summary>
        /// Gets or sets the display order
        /// </summary>
        public virtual int SortOrder { get; set; }
        #endregion

        #region Load Methods
        /// <summary>
        /// Load contact type by name
        /// </summary>
        /// <param name="name">Name of the contact type</param>
        /// <param name="showDeleted">Show deleted</param>
        /// <returns>Contact type by name</returns>
        public static ContactType Load(string name, bool showDeleted)
        {
            Parameters parameters = new Parameters();
            parameters.Add(new Parameter("Name", name));
            parameters.Add(new Parameter("Deleted", showDeleted));
            return DataAccess.GetSingle<ContactType>("LoadContactType", System.Data.CommandType.StoredProcedure, parameters);
        }

        /// <summary>
        /// Load contact type by unique identifier
        /// </summary>
        /// <param name="guid">Unqiue identifier</param>
        /// <param name="showDeleted">Show deleted</param>
        /// <returns>Contact type by unique identifier</returns>
        public static ContactType Load(Guid guid, bool showDeleted)
        {
            Parameters parameters = new Parameters();
            parameters.Add(new Parameter("Guid", guid));
            parameters.Add(new Parameter("Deleted", showDeleted));
            return DataAccess.GetSingle<ContactType>("LoadContactType", System.Data.CommandType.StoredProcedure, parameters);
        }
        #endregion

        #region Save Methods
        /// <summary>
        /// Save contact type to database
        /// </summary>
        public virtual void Save()
        {
            DataAccess.NonQuery("SaveContactType", System.Data.CommandType.StoredProcedure, GetSaveParameters());
        }
        #endregion

        #region Delete Methods
        /// <summary>
        /// Delete contact type from database
        /// </summary>
        public virtual void Delete()
        {
            Parameters parameters = new Parameters();
            parameters.Add(new Parameter("Guid", this.Guid));
            DataAccess.NonQuery("DeleteContactType", System.Data.CommandType.StoredProcedure, parameters);
        }
        #endregion

        #region Fill Method
        /// <summary>
        /// Fill a contact type object with data from a database
        /// </summary>
        /// <param name="reader">Database reader</param>
        public virtual void Fill(System.Data.IDataReader reader)
        {
            this.Guid = Helper.ToGuid(reader["Guid"]);
            this.Name = Helper.ToString(reader["Name"]);
            this.SortOrder = Helper.ToInt(reader["SortOrder"]);
        }
        #endregion

        #region Get Save Parameters
        /// <summary>
        /// Get save parameters
        /// </summary>
        /// <returns>Parameter collection</returns>
        public Parameters GetSaveParameters()
        {
            Parameters parameters = new Parameters();
            parameters.Add(new Parameter("Guid", this.Guid));
            parameters.Add(new Parameter("Name", this.Name));
            parameters.Add(new Parameter("SortOrder", this.SortOrder));
            return parameters;
        }
        #endregion
    }
}
